//
//  CWJourneyDetailCell.m
//  ResaISkane
//
//  Copyright 2009-2010 Fredrik Olsson. All rights reserved.
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License. 
//  You may obtain a copy of the License at 
// 
//  http://www.apache.org/licenses/LICENSE-2.0 
//  
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS, 
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
//

#import "CWJourneyDetailCell.h"
#import "NSDate+CWExtentions.h"
#import "UIColor+CWEditableColor.h"

@implementation CWJourneyDetailCell

-(void)localizeTextLabels;
{
  fromTitleLabel.text = NSLocalizedString(@"FromTitle", nil);
  toTitleLabel.text = NSLocalizedString(@"ToTitle", nil);
  [fromTitleLabel sizeToFit];
  [toTitleLabel sizeToFit];
  CGFloat maxTitleWidth = MAX(fromTitleLabel.bounds.size.width, toTitleLabel.bounds.size.width);
  CGRect bounds = fromTitleLabel.bounds;
  bounds.size.width = maxTitleWidth;
  fromTitleLabel.bounds = bounds;
  bounds = toTitleLabel.bounds;
  bounds.size.width = maxTitleWidth;
  toTitleLabel.bounds = bounds;  
  towardsTitleLabel.text = NSLocalizedString(@"TowardsTitle", nil);
}

+(CWJourneyDetailCell*)journeyDetailCell;
{
  NSArray* objects = [[NSBundle mainBundle] loadNibNamed:@"CWJourneyDetailCell" owner:self options:nil];
  for (id object in objects) {
  	if ([object tag] == 1) {
			[object localizeTextLabels];
			return object;
    }
  }
  return nil;
}


-(CGFloat)rowHeight;
{
	if (deviationsLabel.hidden) {
  	return 108.f;
  } else {
		return 108.f + deviationsLabel.frame.size.height;
  }
}

-(void)setRouteLink:(CWRouteLink*)routeLink;
{
  lineTypeImageView.image = [UIImage imageNamed:[NSString stringWithFormat:@"type-%d.png", routeLink.line.type]];
  NSString* line = routeLink.line.typeName;
  if (routeLink.line.number != nil) {
  	line = [line stringByAppendingFormat:@" %@", routeLink.line.number];  
  }
  lineLabel.text = line;
  stopPointLabel.text = [NSString stringWithFormat:NSLocalizedString([routeLink.line isTrain] ? @"Platform" : @"Position", nil), routeLink.from.stopPoint];
	fromLabel.text = routeLink.from.name;
  toLabel.text = routeLink.to.name;
	fromTimeLabel.text = [routeLink.actualDeparture localizedShortTimeString];
  fromTimeLabel.textColor = [routeLink.departure isEqualToDate:routeLink.actualDeparture] ? [UIColor editableColor] : [UIColor warningTextColor];
	toTimeLabel.text = [routeLink.actualArrival localizedShortTimeString];
  toTimeLabel.textColor = [routeLink.arrival isEqualToDate:routeLink.actualArrival] ? [UIColor editableColor] : [UIColor warningTextColor];
	towardsLabel.text = routeLink.line.towards;
  NSString* deviationsString = routeLink.deviationsAsString;
  if (deviationsString) {
  	deviationsLabel.text = [deviationsString stringByAppendingString:@"\n"];
    deviationsLabel.hidden = NO;
  } else {
  	deviationsLabel.hidden = YES;
  }
  [self setNeedsLayout];
}


@end
