//
//  CWLine.m
//  ResaISkane
//
//  Copyright 2009-2010 Fredrik Olsson. All rights reserved.
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License. 
//  You may obtain a copy of the License at 
// 
//  http://www.apache.org/licenses/LICENSE-2.0 
//  
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS, 
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
//

#import "CWLine.h"


@implementation CWLine

@synthesize name = _name;
@synthesize number = _number;
@synthesize type = _type;
@synthesize towards = _towards;

-(NSString*)typeName;
{
	if (_type == CWLineTypeGang) {
  	return NSLocalizedString(@"Walk", nil);
  }
  return _typeName;
}

-(id)initWithCoder:(NSCoder *)aDecoder;
{
  self = [self init];
  if (self) {
    _name = [[aDecoder decodeObjectForKey:@"name"] retain];
    _number = [[aDecoder decodeObjectForKey:@"number"] retain];
    _type = [aDecoder decodeIntegerForKey:@"type"];
    _typeName = [[aDecoder decodeObjectForKey:@"typeName"] retain];
    _towards = [[aDecoder decodeObjectForKey:@"towards"] retain];
  }
  return self;
}


-(BOOL)isTrain;
{
	switch (_type) {
    case CWLineTypeOresundsTag:
    case CWLineTypePagatagenTag:
			return YES;
    default:
			return NO;
  }
}

-(void)encodeWithCoder:(NSCoder *)aCoder;
{
  [aCoder encodeObject:self.name forKey:@"name"];
  [aCoder encodeObject:self.number forKey:@"number"];
  [aCoder encodeInteger:self.type forKey:@"type"];
  [aCoder encodeObject:self.typeName forKey:@"typeName"];
  [aCoder encodeObject:self.towards forKey:@"towards"];
}

-(void)dealloc;
{
	[_name release]; 
  [_number release]; 
	[_typeName release]; 
  [_towards release];
  [super dealloc];
}

-(BOOL)isEqual:(id)anObject;
{
  if (self == anObject) {
    return YES;
  } else if ([anObject isKindOfClass:[CWLine class]]) {
    CWLine* otherLine = anObject;
    return ([self.name isEqualToString:otherLine.name] && 
            [self.number isEqualToString:otherLine.number] &&
            self.type == otherLine.type &&
            [self.towards isEqualToString:otherLine.towards]);
  }
  return NO;
}

-(NSString*)description;
{
  return [NSString stringWithFormat:@"<CWLine: '%@', %@, %@(%d), '%@'>", self.name, self.number, self.typeName, self.type, self.towards];
}

@end
