//
//  CWStopSelectionController.h
//  ResaISkane
//
//  Copyright 2009-2010 Fredrik Olsson. All rights reserved.
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License. 
//  You may obtain a copy of the License at 
// 
//  http://www.apache.org/licenses/LICENSE-2.0 
//  
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS, 
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
//

#import <UIKit/UIKit.h>
#import "CWModel.h"

@protocol CWPointSelectionControllerDelegate;

/*!
 * @abstract Type of point selection.
 */
typedef enum {
  CWPointSelectionTypeFrom,  //! From point.
  CWPointSelectionTypeTo     //! To point.
} CWPointSelectionType;

/*!
 * @abstract Controller responsible for selecting a point to travek from or to, including as -you-type search.
 *
 * @discussion The controller will add selected point as a known point to the shred model if it is not previosule known.
 */
@interface CWPointSelectionController : UITableViewController <UISearchDisplayDelegate> {
@private
  CWModel* sharedModel;
	id<CWPointSelectionControllerDelegate> pointSelectionDelegate;
	BOOL searchOnly;
	CWPointSelectionType _pointType;
	NSOperationQueue* queue;
  NSArray* _searchResultPoints;
}

@property(nonatomic, readonly, assign) CWPointSelectionType pointType; //! Type of point selection.
@property(nonatomic, assign) BOOL searchOnly;                          //! YES if controller should start in search mode.

/*!
 * @abstract Iit controller with delegate to call for user actions.
 */
-(id)initWithPointSelectionType:(CWPointSelectionType)pointType delegate:(id<CWPointSelectionControllerDelegate>)delegate;

@end


/*!
 * @abstract Delegate protocol for responding to user actions.
 */
@protocol CWPointSelectionControllerDelegate <NSObject>

/*!
 * @abstract User selected a point.
 *
 * @discussion It is the responsiblity of the delegate to dismiss the controller if needed.
 */
-(void)pointSelectionController:(CWPointSelectionController*)controller didSelectPoint:(CWPoint*)point;

@end
